#!/usr/bin/python
import bz2
import wave
import time
import os

class nmea_analyzer:
  def __init__(self):
    self.latitude=0
    self.longitude=0
    self.altitude=0
    self.timeOfFix=""
    self.line=""
    return
  def put(self,c):
    #Add the character to the line
    self.line+=c
    #If end of line has been reached, check the sentence
    if self.line[-2:]=="\r\n":
       #If this is GPGGA, parse it
       #code based on: http://snippets.dzone.com/posts/show/4782
       if self.line[0:6]=="$GPGGA":
	try:
         (self.format,
          self.utc,
          self.latitude, 
	  self.northsouth, 
	  self.longitude, 
	  self.eastwest, 
	  self.quality, 
	  self.number_of_satellites_in_use, 
	  self.horizontal_dilution, 
	  self.altitude, 
	  self.above_sea_unit, 
	  self.geoidal_separation, 
	  self.geoidal_separation_unit, 
	  self.data_age, 
	  self.diff_ref_stationID) = self.line.split(",")
  	 latitude_in=float(self.latitude)
	 longitude_in=float(self.longitude)
	 if self.northsouth == 'S':
           latitude_in = -latitude_in
	 if self.eastwest == 'W':
           longitude_in = -longitude_in
	 latitude_degrees = int(latitude_in/100)
	 latitude_minutes = latitude_in - latitude_degrees*100
	 longitude_degrees = int(longitude_in/100)
	 longitude_minutes = longitude_in - longitude_degrees*100
	 self.latitude = latitude_degrees + (latitude_minutes/60)
	 self.longitude = longitude_degrees + (longitude_minutes/60)
	 self.timeOfFix = time.strftime("%H:%M:%S", time.strptime(self.utc.split(".")[0],"%H%M%S"))
	 self.altitude = float(self.altitude)
	except:
         pass
       self.line=""
    return
  def close(self):
    return

class voice_output:
  def __init__(self):
    self.in_msg=False
    self.fcnt=0
    self.vsmp=0
    return
  def put(self,c):
    if not self.in_msg:
	#We need to create a new sound file
	self.fname="v"+str(self.fcnt)+".wav"
        self.fout=wave.open(self.fname,"w")
	self.fout.setnchannels(1)
	self.fout.setsampwidth(2)
	self.fout.setframerate(8000)
	self.fcnt+=1
        self.in_msg = True
	#We need to output the waypoint
        s="<wpt lat=\""+str(nmea_a.latitude)+"\" lon=\""+str(nmea_a.longitude)+"\">\n"
        s+="<ele>"+str(nmea_a.altitude)+"</ele>\n"
        s+="<time>"+nmea_a.timeOfFix+"</time>\n"
        s+="<link href=\""+self.fname+"\" />\n"
        s+="</wpt>\n"
	fwpt.write(s)
    #Now we can output the sample
    if c & 0x40:
       self.vsmp |= (c & 0x3f)*64;
    else:
       self.vsmp |= (c & 0x3f)
       self.vsmp *= 16
       self.vsmp ^= 0x8000 ; #conversion to signed...
       self.fout.writeframesraw(chr(self.vsmp & 255)+chr(self.vsmp / 256))
       self.vsmp=0
    return
  def stop(self):
    self.in_msg=False
    return
  def active(self):
    return self.in_msg
  def close(self):
    self.fout.close()
    return

nmea_a = nmea_analyzer()
voice = voice_output()
fnmea=open("nmea.txt","w")
fwpt=open("wpt.gpx","w")
fwpt.write("<?xml version='1.0' encoding='UTF-8' ?>\n")
fwpt.write("<gpx version=\"1.1\" creator=\"my_conv\" xmlns=\"http://www.topografix.com/GPX/1/1\" xmlns:rmc=\"urn:net:trekbuddy:1.0:nmea:rmc\">")

f=bz2.BZ2File("dump.bin.bz2","rb")
#Now we should read the data from the file,
#track the location and detect the messages...
nmea_cnt = 0
while True:
  c=f.read(1)
  if c=="":
	break
  else:
     if ord(c)<128:
	fnmea.write(c)
        if voice.active():
          nmea_cnt+=1
	  if nmea_cnt > 2:
	     voice.stop()
             nmea_cnt=0
        nmea_a.put(c)
     else:
	nmea_cnt=0
	voice.put(ord(c))
nmea_a.close()
voice.close()
fwpt.write("</gpx>\n")
fwpt.close()
fnmea.close()
os.system("gpsbabel -i nmea -f nmea.txt -o gpx -F track.gpx")

